import os
import logging
from importlib import import_module
from .. import GLUE_PLATE_ENVIRONMENT_VARIABLE_KEY, GLUE_PLATE_PARENT_MODULES


logger = logging.getLogger(__name__)

ENVIRONMENT_VARIABLE = 'GLUE_PLATE_MODULE'

def _merge_parents(settings):
    if GLUE_PLATE_PARENT_MODULES in settings:
        for parent_module_name in settings.GLUE_PLATE_PARENT_MODULES:
            parent_settings = import_module(parent_module_name).settings
            if GLUE_PLATE_ENVIRONMENT_VARIABLE_KEY in parent_settings:
                if parent_settings.GLUE_PLATE_ENVIRONMENT_VARIABLE_KEY in os.environ:
                    parent_child = os.environ[parent_settings.GLUE_PLATE_ENVIRONMENT_VARIABLE_KEY]
                    _child = import_module(parent_child).settings
                    parent_settings.update(_child)
                    if GLUE_PLATE_PARENT_MODULES in parent_settings:
                        parent_settings = _merge_parents(parent_settings)
            parent_settings.update(settings)
            settings = parent_settings
    return settings

if __name__ == 'glueplate.config':
    if not os.environ.get('GLUE_PLATE_BASE_MODULE', None):
        raise ValueError("need glueplate's GLUE_PLATE_BASE_MODULE in environment.")
    base_module_name = os.environ['GLUE_PLATE_BASE_MODULE']
    logger.info('Load base glueplate config: {0}'.format(base_module_name))
    settings = import_module(base_module_name).settings
    if GLUE_PLATE_ENVIRONMENT_VARIABLE_KEY not in settings:
        raise ValueError("neet GLUE_PLATE_ENVIRONMENT_VARIABLE_KEY in base config settings.")
    settings = _merge_parents(settings)
    modified_module_name = os.environ[settings.GLUE_PLATE_ENVIRONMENT_VARIABLE_KEY]
    _settings = import_module(modified_module_name).settings
    settings.update(_settings)



import collections

import logging
logger = logging.getLogger(__name__)

GLUE_PLATE_ENVIRONMENT_VARIABLE_KEY = 'GLUE_PLATE_ENVIRONMENT_VARIABLE_KEY'
GLUE_PLATE_PLUS_BEFORE = 'GLUE_PLATE_PLUS_BEFORE_'
GLUE_PLATE_PLUS_AFTER = 'GLUE_PLATE_PLUS_AFTER_'
GLUE_PLATE_PARENT_MODULES = 'GLUE_PLATE_PARENT_MODULES'

def _update(org, opt):
    for k, v in opt.items():
        if k.startswith(GLUE_PLATE_PLUS_BEFORE) and isinstance(v, list):
            _k = k[len(GLUE_PLATE_PLUS_BEFORE):]
            org[_k] = v + org[_k]
        elif k.startswith(GLUE_PLATE_PLUS_AFTER) and isinstance(v, list):
            _k = k[len(GLUE_PLATE_PLUS_AFTER):]
            org[_k] = org[_k] + v
        elif isinstance(v, collections.Mapping):
            r = _update(org.get(k, Glue()), v)
            org[k] = r
        else:
            org[k] = opt[k]
    return org


class Glue(dict):
    '''dict like object. accessible with dot syntax.
    >>> settings = Glue(
    ...   spam = '123',
    ...   egg = 123,
    ...   lst = [1,2,3],
    ... )
    >>> assert(settings.spam == '123')
    >>> assert(settings.egg == 123)
    >>> assert(settings.lst == [1,2,3])
    >>> settings.ham = 123.0
    >>> assert(settings.ham == 123.0)
    >>> s2 = Glue(dict(spam=456))
    >>> settings.update(s2)
    >>> assert(settings.spam == 456)
    >>> assert(settings.ham == 123.0)
    >>> s3 = Glue(GLUE_PLATE_PLUS_BEFORE_lst=[0])
    >>> settings.update(s3)
    >>> assert(settings.lst == [0,1,2,3])
    >>> s4 = Glue(GLUE_PLATE_PLUS_AFTER_lst=[4])
    >>> settings.update(s4)
    >>> assert(settings.lst == [0,1,2,3,4])
    '''

    def __init__(self, *args, **kwargs):
        for d in args:
            if isinstance(d, collections.Mapping):
                self.update(d)
        for key, value in kwargs.items():
            self[key] = value

    def __setattr__(self, key, value):
        if isinstance(value, collections.Mapping):
            self[key] = Glue(value)
        else:
            self[key] = value

    def __getattr__(self, key):
        try:
            return self[key]
        except:
            object.__getattribute__(self, key)

    def update(self, opt):
        self = _update(self, opt)
